#include <iostream>
#include <vector>
using namespace std;

struct Input {
    vector<double> numbers;
    size_t bin_count{};
};

Input
input_data() {
    cerr << "Enter number count, numbers and bin count: ";

    size_t number_count;
    cin >> number_count;

    Input in;
    in.numbers.resize(number_count);

    for (size_t i = 0; i < number_count; i++) {
        cin >> in.numbers[i];
    }

    cin >> in.bin_count;

    return in;
}

void
find_minmax(const vector<double>& numbers, double& min, double& max) {
    min = numbers[0];
    max = numbers[0];

    for (double x : numbers) {
        if (x < min) {
            min = x;
        } else if (x > max) {
            max = x;
        }
    }
}

vector<size_t>
make_histogram(const vector<double>& numbers, size_t bin_count) {
    vector<size_t> bins(bin_count, 0);

    double min, max;
    find_minmax(numbers, min, max);

    double bin_size = (max - min) / bin_count;
    if (bin_size == 0) {
        bin_size = 1;
    }

    for (size_t i = 0; i < numbers.size(); i++) {
        bool found = false;
        for (size_t j = 0; (j < bin_count - 1) && !found; j++) {
            auto lo = min + j * bin_size;
            auto hi = min + (j + 1) * bin_size;
            if ((lo <= numbers[i]) && (numbers[i] < hi)) {
                bins[j]++;
                found = true;
            }
        }
        if (!found) {
            bins[bin_count - 1]++;
        }
    }

    return bins;
}

void
show_histogram_text(const vector<size_t>& bins, size_t screen_width = 80) {
    const size_t MAX_ASTERISK = screen_width - 3 - 1;

    size_t max_count = 0;
    for (size_t count : bins) {
        if (count > max_count) {
            max_count = count;
        }
    }

    for (size_t i = 0; i < bins.size(); i++) {
        size_t height = 0;

        if (max_count > MAX_ASTERISK) {
            height = static_cast<size_t>(
                MAX_ASTERISK * static_cast<double>(bins[i]) / max_count
            );
        } else {
            height = bins[i];
        }

        if (bins[i] < 10) {
            cout << "  " << bins[i] << "|";
        } else if (bins[i] < 100) {
            cout << " " << bins[i] << "|";
        } else {
            cout << bins[i] << "|";
        }

        for (size_t j = 0; j < height; j++) {
            cout << "*";
        }
        cout << endl;
    }
}

int
main() {
    auto in = input_data();
    auto bins = make_histogram(in.numbers, in.bin_count);
    show_histogram_text(bins);

    return 0;
}
